"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emptyRelatedModule = exports.placeholderModuleLink = exports.emptyModuleLink = exports.relatedModule = exports.moduleAssociation = exports.moduleLink = void 0;
const bf_types_1 = require("bf-types");
const Strings_1 = require("../Strings");
const EXTRA_MODULE_NAMES = Object.freeze(['PLACEHOLDER']);
const VALID_MODULE_NAMES = Object.freeze([...EXTRA_MODULE_NAMES, ...Object.values(bf_types_1.ALL_MODULES)]);
/**
 * Create a new module link object
 *
 * This function provides type safety for the moduleName parameter. It ensures that
 * the module name is one the allowed module names. Because of that, it should be used whenever
 * a new module link is needed.
 *
 * @param moduleName The name of the module this link references
 * @param moduleId The ID of the Entity in the module this link references
 */
function moduleLink(moduleName, moduleId) {
    return {
        module_name: ensureValidModuleName(moduleName),
        module_id: moduleId,
    };
}
exports.moduleLink = moduleLink;
/**
 * Create a new module association object
 *
 * This function provides type safety for the moduleName parameter. Because of that, it should
 * be used whenever a new module association is needed.
 *
 * @param moduleName The name of the module this association references
 * @param moduleId The ID of the Entity in the module this association references
 * @param id An optional ID to specify for this module association
 */
function moduleAssociation(moduleName, moduleId, id = '') {
    return {
        id,
        module_name: ensureValidModuleName(moduleName),
        module_id: moduleId,
    };
}
exports.moduleAssociation = moduleAssociation;
/**
 * Create a new related module object
 *
 * This function provides type safety for the moduleName parameter. Because of that, it should
 * be used whenever a new related module is needed.
 *
 * @param moduleName The name of the module this related module object references
 * @param moduleId The ID of the Entity in the module this module object references
 * @param moduleTitle The title of the Entity in the module this module object references
 */
function relatedModule(moduleName, moduleId, moduleTitle) {
    return {
        module_name: ensureValidModuleName(moduleName),
        module_id: moduleId,
        module_title: moduleTitle,
    };
}
exports.relatedModule = relatedModule;
/**
 * Creates a new module link with an empty string for each field
 */
function emptyModuleLink() {
    return { module_name: '', module_id: '' };
}
exports.emptyModuleLink = emptyModuleLink;
/**
 * Creates a new module link that is meant to be used as a placeholder
 */
function placeholderModuleLink(moduleId = '') {
    return { module_name: 'PLACEHOLDER', module_id: moduleId };
}
exports.placeholderModuleLink = placeholderModuleLink;
/**
 * Creates a new related module with an empty string for each field
 */
function emptyRelatedModule() {
    return { module_name: '', module_id: '', module_title: '' };
}
exports.emptyRelatedModule = emptyRelatedModule;
function ensureValidModuleName(moduleName) {
    const sanitizedModuleName = Strings_1.toUpperSnakeCase(moduleName);
    if (!VALID_MODULE_NAMES.includes(sanitizedModuleName)) {
        throw new Error(`${sanitizedModuleName} is not allowed to be used as a module name.`);
    }
    return sanitizedModuleName;
}
