"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const io = require("socket.io-client");
const msgpackParser = require("socket.io-msgpack-parser");
const system_1 = require("../system");
const Utils_1 = require("../common/Utils");
const Types_1 = require("./Types");
class LiveSyncException extends Error {
    constructor(message) {
        super(message);
    }
}
function subscribeAfterBegun() {
    throw new LiveSyncException("Can't subscribe after the connection has already begun.");
}
function noSubscriptionSet(eventType) {
    throw new LiveSyncException(`Failed to unsubscribe to the ${eventType} event type. ${eventType} has no subscription set.`);
}
function subscriptionNotInSet(eventType) {
    const msg = `Failed to unsubscribe to the ${eventType} event type.
  The provided subscription is not a part of the ${eventType} subscription set.`;
    throw new LiveSyncException(msg);
}
const AUTHENTICATION = 'authentication';
const subscriptionMap = new Map();
let socket;
let isSyncing = false;
let hasBegun = false;
let connectionTries = 0;
function begin() {
    if (isSyncing) {
        // TODO: Myabe add some form of logging here
        return;
    }
    system_1.default.getEventBus().subscribe(Types_1.LIVE_SYNC, handleInternalPublishEvent);
    const connectionOptions = system_1.default.liveSyncOptions();
    if (!connectionOptions) {
        // TODO: Myabe add some form of logging here
        return;
    }
    isSyncing = true;
    const baseUrl = system_1.default.nexus.getUrl();
    socket = io(baseUrl, {
        transports: ['websocket'],
        autoConnect: false,
        parser: msgpackParser,
    });
    socket.on(Types_1.ConnectionEventType.CONNECT, () => {
        socket.emit(AUTHENTICATION, connectionOptions);
        invokeSubscriptionSet(Types_1.ConnectionEventType.CONNECT, null);
    });
    socket.on(Types_1.ConnectionEventType.DISCONNECT, reconnect);
    socket.on(Types_1.ConnectionEventType.UNAUTHORIZED, (reason) => {
        invokeSubscriptionSet(Types_1.ConnectionEventType.UNAUTHORIZED, reason);
        socket.disconnect();
    });
    for (const eventType of subscriptionMap.keys()) {
        socket.on(eventType, (data) => {
            invokeSubscriptionSet(eventType, data);
        });
    }
    socket.open();
    hasBegun = true;
}
function subscribe(eventType, subscription) {
    let subscriptionSet = subscriptionMap.get(eventType);
    if (!subscriptionSet) {
        if (hasBegun) {
            subscribeAfterBegun();
        }
        subscriptionSet = new Set();
        subscriptionMap.set(eventType, subscriptionSet);
    }
    subscriptionSet.add(subscription);
    return subscription;
}
function unsubscribe(eventType, subscription) {
    const subscriptionSet = subscriptionMap.get(eventType);
    if (!subscriptionSet) {
        noSubscriptionSet(eventType);
    }
    if (!subscriptionSet.has(subscription)) {
        subscriptionNotInSet(eventType);
    }
    subscriptionSet.delete(subscription);
    return subscription;
}
/* ~~ Helper Functions ~~ */
function handleInternalPublishEvent(event) {
    if (event.topic === Types_1.PUBLISH_LIVE_SYNC_EVENT) {
        invokeSubscriptionSet(event.data.eventType, event.data.eventPayload);
    }
}
/**
 * Invoke an entire subscription set with data emitted from an event
 *
 * @param eventType The globally unique name for the event the data was emitted from
 * @param data The data that will be passed as an argument to each subscription
 */
function invokeSubscriptionSet(eventType, data) {
    const subscriptionSet = subscriptionMap.get(eventType);
    if (subscriptionSet) {
        for (const subscription of subscriptionSet) {
            subscription(data);
        }
    }
}
function reconnect() {
    return __awaiter(this, void 0, void 0, function* () {
        isSyncing = false;
        socket.close();
        if (connectionTries > 10) {
            yield invokeSubscriptionSet(Types_1.ConnectionEventType.DISCONNECT, null);
        }
        else {
            yield Utils_1.sleep(1000);
            connectionTries++;
            begin();
        }
    });
}
const livesync = {
    begin,
    subscribe,
    unsubscribe,
};
exports.default = system_1.default.sealModule(Object.freeze(livesync));
